@@ -555,7 +555,7 @@ class MarkCriterion(Criterion):
         return self._set_polarity(True)
 
     def __mark2str(self):
-        """Convert the mark/mask to a string
+        """Convert the mark/mask to a string; both values will be in hex
         """
         markstr = f'{self.__mark:#x}'
         if self.__mask is not None:
