@@ -1,4 +1,4 @@
-# Copyright (c) 2021, 2022, Panagiotis Tsirigotis
+# Copyright (c) 2021, 2022, 2023, Panagiotis Tsirigotis
 
 # This file is part of linuxnet-iptables.
 #
@@ -277,65 +277,184 @@ class RejectTarget(Target):
         return RejectTarget(reject_with=icmp_message)
 
 
-class MarkTarget(Target):
-    """This class provides access to the ``MARK`` target
+
+class _MarkOperations:
+    """Mixin class to provide mark-related operations
     """
-    def __init__(self, mark: Optional[int] =None, xmark: Optional[int] =None):
-        """
-        :param mark: value used to set the mark value in the packet
-        :param xark: see **iptables(8)** **MARK** target
-        """
-        super().__init__('MARK', terminates=False)
-        self.__mark = mark
-        self.__xmark = xmark
+
+    #
+    # Operations
+    #
+    SET = 1
+    XSET = 2
+    AND = 3
+    OR = 4
+    XOR = 5
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.__value = None
+        self.__mask = None
+        self.__op = None
 
     def get_mark(self) -> Optional[int]:
         """Returns the mark value set by this target
         """
-        return self.__mark
+        return self.__value
+
+    def get_mask(self) -> Optional[int]:
+        """Returns the mask used by this target
+        """
+        return self.__mask
+
+    def get_op(self) -> Optional[int]:
+        """Returns the operation
+
+        :rtype: an integer with one of the following values:
+                `SET`, `XSET`, `AND`, `OR`, `XOR`,
+                or ``None``
+        """
+        return self.__op
+
+    def set_mark(self, value: int, mask: Optional[int] =None) -> Target:
+        """Set the mark
+        """
+        if self.__op is not None:
+            raise IptablesError(f'mark operation already set: {self.__op}')
+        self.__value = value
+        self.__mask = mask
+        self.__op = self.SET
+        return self
+
+    def set_xmark(self, value: int, mask: Optional[int] =None) -> Target:
+        """Alternative way of setting the mark
+        """
+        if self.__op is not None:
+            raise IptablesError(f'mark operation already set: {self.__op}')
+        self.__value = value
+        self.__mask = mask
+        self.__op = self.XSET
+        return self
+
+    def and_mark(self, mask: int) -> Target:
+        """Clear the bits identified by mask
+        """
+        if self.__op is not None:
+            raise IptablesError(f'mark operation already set: {self.__op}')
+        self.__value = None
+        self.__mask = mask
+        self.__op = self.AND
+        return self
+
+    def or_mark(self, mask) -> Target:
+        """Set the bits identified by mask
+        """
+        if self.__op is not None:
+            raise IptablesError(f'mark operation already set: {self.__op}')
+        self.__value = None
+        self.__mask = mask
+        self.__op = self.OR
+        return self
+
+    def xor_mark(self, mask) -> Target:
+        """Xor the bits identified by mask
+        """
+        if self.__op is not None:
+            raise IptablesError(f'mark operation already set: {self.__op}')
+        self.__value = None
+        self.__mask = mask
+        self.__op = self.XOR
+        return self
+
+    def parse_op(self, val: str, field_iter) -> bool:
+        """Parse the operation identified by 'val'
+        """
+        if val == 'set':
+            self.set_mark(int(field_iter.next_value(val), 16))
+        elif val == 'xset':
+            field = field_iter.next_value(val)
+            valstr, maskstr = field.split('/', 1)
+            self.set_xmark(int(valstr, 16), int(maskstr, 16))
+        elif val == 'or':
+            self.or_mark(int(field_iter.next_value(val), 16))
+        elif val == 'xor':
+            self.xor_mark(int(field_iter.next_value(val), 16))
+        elif val == 'and':
+            self.and_mark(int(field_iter.next_value(val), 16))
+        else:
+            return False
+        return True
+
+    def mark_iptables_args(self, args: List[str]) -> List[str]:
+        """Converts the op/value/mask to a list of **iptables(8)** arguments
+        for the MARK target
+        """
+        if self.__value is None and self.__mask is None:
+            raise IptablesError("no mark operation specified")
+        if self.__op in (self.SET, self.XSET):
+            if self.__op == self.SET:
+                args.append('--set-mark')
+            else:
+                args.append('--set-xmark')
+            if self.__mask is None:
+                args.append(f'0x{self.__value:x}')
+            else:
+                args.append(f'0x{self.__value:x}/0x{self.__mask:x}')
+        elif self.__op == self.AND:
+            args.extend(['--and-mark', f'0x{self.__mask:x}'])
+        elif self.__op == self.OR:
+            args.extend(['--or-mark', f'0x{self.__mask:x}'])
+        elif self.__op == self.XOR:
+            args.extend(['--xor-mark', f'0x{self.__mask:x}'])
+        else:
+            raise IptablesError(f"unexpected mark operation: {self.__op}")
+        return args
+
+
+class MarkTarget(_MarkOperations, Target):
+    """This class provides access to the ``MARK`` target
+    """
+    def __init__(self, mark: Optional[int] =None):
+        """
+        :param mark: value used to set the mark value in the packet
+        """
+        super().__init__('MARK', terminates=False)
+        if mark is not None:
+            self.set_mark(mark)
 
     def to_iptables_args(self) -> List[str]:
         """Returns a list of **iptables(8)** arguments
         """
         retval = super().to_iptables_args()
-        if self.__mark is not None:
-            retval += ['--set-mark', f'0x{self.__mark:x}']
-        if self.__xmark is not None:
-            retval += ['--set-xmark', f'0x{self.__xmark}']
-        return retval
+        return self.mark_iptables_args(retval)
 
     @classmethod
     def _parse(cls, field_iter) -> Target:
         """Parse the MARK target options
         """
         field_iter.forward_past('MARK')
-        mark = None
-        xmark = None
+        target = MarkTarget()
         for val in field_iter:
-            if val == 'set':
-                mark = int(field_iter.next_value(val), 16)
-            elif val == 'xset':
-                xmark = int(field_iter.next_value(val), 16)
-            else:
+            if not target.parse_op(val, field_iter):
                 raise IptablesParsingError(f'unknown MARK argument: {val}')
-        return MarkTarget(mark=mark, xmark=xmark)
+        return target
 
 
-class ConnmarkTarget(Target):
+class ConnmarkTarget(_MarkOperations, Target):
     """This class provides access to the ``CONNMARK`` target
     """
-    def __init__(self, *, mark: Optional[int] =None,
-                        xmark: Optional[int] =None, restore_mark=False):
+    def __init__(self, *, mark: Optional[int] =None, restore_mark=False):
         """
         :param mark: value used to set the ctmark value (associated with
             a connection)
-        :param xark: see **iptables(8)** **CONNMARK** target
         :param restore_mark: if ``True``, copy the connection mark to the
             packet mark
         """
         super().__init__('CONNMARK', terminates=False)
-        self.__mark = mark
-        self.__xmark = xmark
+        if mark is not None:
+            if restore_mark:
+                raise IptablesError('can either set or restore mark, not both')
+            self.set_mark(mark)
         self.__restore_mark = restore_mark
 
     def to_iptables_args(self) -> List[str]:
@@ -344,30 +463,19 @@ class ConnmarkTarget(Target):
         retval = super().to_iptables_args()
         if self.__restore_mark:
             retval += ['--restore-mark']
-        if self.__mark is not None:
-            retval += ['--set-mark', str(self.__mark)]
-        if self.__xmark is not None:
-            retval += ['--set-xmark', str(self.__xmark)]
-        return retval
-
-    def get_mark(self) -> Optional[int]:
-        """Returns the mark to set
-        """
-        return self.__mark
-
-    def get_xmark(self) -> Optional[int]:
-        """Returns the xmark to set
-        """
-        return self.__xmark
+            return retval
+        return self.mark_iptables_args(retval)
 
     def is_restoring_mark(self) -> bool:
-        """Returns True if we are restoring the mark
+        """Returns ``True`` if we are restoring the mark
         """
         return self.__restore_mark
 
     def restore_mark(self) -> None:
         """Sets the restore mark flag
         """
+        if self.get_op() is not None:
+            raise IptablesError('mark setting operation already specified')
         self.__restore_mark = True
 
     @classmethod
@@ -375,21 +483,15 @@ class ConnmarkTarget(Target):
         """Parse the CONNMARK target options
         """
         field_iter.forward_past('CONNMARK')
-        mark = None
-        xmark = None
-        restore_mark = False
+        target = ConnmarkTarget()
         for val in field_iter:
-            if val == 'set':
-                mark = int(field_iter.next_value(val), 16)
-            elif val == 'xset':
-                xmark = int(field_iter.next_value(val), 16)
+            if target.parse_op(val, field_iter):
+                pass
             elif val == 'restore':
-                restore_mark = True
+                target.restore_mark()
             else:
-                raise IptablesParsingError(
-                            f'unknown CONNMARK argument: {val}')
-        return ConnmarkTarget(mark=mark, xmark=xmark,
-                                        restore_mark=restore_mark)
+                raise IptablesParsingError(f'unknown CONNMARK argument: {val}')
+        return target
 
 
 class _NatTarget(Target):
