@@ -1,4 +1,4 @@
-# Copyright (c) 2022, Panagiotis Tsirigotis
+# Copyright (c) 2022, 2023, Panagiotis Tsirigotis
 
 # This file is part of linuxnet-iptables.
 #
@@ -36,7 +36,7 @@ else:
 
 from linuxnet.iptables import IptablesPacketFilterTable
 from linuxnet.iptables import IptablesError
-from linuxnet.iptables import ChainRule, ChainTarget, Targets
+from linuxnet.iptables import ChainRule, ChainTarget, Targets, MarkTarget
 from linuxnet.iptables import CommentMatch
 from linuxnet.iptables import TcpmssMatch, TcpMatch
 from linuxnet.iptables import UdpMatch, IcmpMatch
@@ -467,6 +467,53 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         self.assertEqual(target.get_rejection_message(),
                                 'icmp-host-unreachable')
 
+    def test_parsing_mark_target(self):
+        """Parse output with MARK target
+        """
+        output = """\
+Chain INPUT (policy DROP 0 packets, 0 bytes)
+    pkts      bytes target     prot opt in     out     source               destination
+    0     0 MARK       all  --  *      *       0.0.0.0/0            0.0.0.0/0           MARK set 0xf 
+    0     0 MARK       all  --  *      *       0.0.0.0/0            0.0.0.0/0           MARK xset 0x11/0xffff0011 
+    0     0 MARK       all  --  *      *       0.0.0.0/0            0.0.0.0/0           MARK xor 0xf 
+    0     0 MARK       all  --  *      *       0.0.0.0/0            0.0.0.0/0           MARK or 0xf 
+    0     0 MARK       all  --  *      *       0.0.0.0/0            0.0.0.0/0           MARK and 0xff 
+""" + '\n' + self.EMPTY_FORWARD + '\n' + self.EMPTY_OUTPUT
+        pft = IptablesPacketFilterTable('filter')
+        self.assertTrue(pft.init_from_output(output))
+        input_chain = pft.get_builtin_chain('INPUT')
+        # Verify SET
+        rule = input_chain.get_rules()[0]
+        target = rule.get_target()
+        self.assertEqual(target, Targets.MARK)
+        self.assertEqual(target.get_op(), MarkTarget.SET)
+        self.assertEqual(target.get_mark(), 0xf)
+        # Verify XSET
+        rule = input_chain.get_rules()[1]
+        target = rule.get_target()
+        self.assertEqual(target, Targets.MARK)
+        self.assertEqual(target.get_op(), MarkTarget.XSET)
+        self.assertEqual(target.get_mark(), 0x11)
+        self.assertEqual(target.get_mask(), 0xffff0011)
+        # Verify XOR
+        rule = input_chain.get_rules()[2]
+        target = rule.get_target()
+        self.assertEqual(target, Targets.MARK)
+        self.assertEqual(target.get_op(), MarkTarget.XOR)
+        self.assertEqual(target.get_mask(), 0xf)
+        # Verify OR
+        rule = input_chain.get_rules()[3]
+        target = rule.get_target()
+        self.assertEqual(target, Targets.MARK)
+        self.assertEqual(target.get_op(), MarkTarget.OR)
+        self.assertEqual(target.get_mask(), 0xf)
+        # Verify AND
+        rule = input_chain.get_rules()[4]
+        target = rule.get_target()
+        self.assertEqual(target, Targets.MARK)
+        self.assertEqual(target.get_op(), MarkTarget.AND)
+        self.assertEqual(target.get_mask(), 0xff)
+
     def test_parsing_connmark_target(self):
         """Parse output with CONNMARK target
         """
@@ -518,6 +565,36 @@ Chain prod_INPUT (1 references)
 
 
 
+class TestTargetGeneration(unittest.TestCase):
+    """Test generation of iptables arguments for targets
+    """
+
+    def test_mark_target_args(self):
+        """iptables argument generation for the MARK target
+        """
+        # test no args
+        target = MarkTarget()
+        self.assertRaises(IptablesError, target.to_iptables_args)
+        # test attempt to double-set
+        target = MarkTarget(10)
+        self.assertRaises(IptablesError, target.and_mark, 0xff)
+        # test set via constructor
+        target = MarkTarget(10)
+        self.assertEqual(target.to_iptables_args(), ['MARK', '--set-mark', '0xa'])
+        # test the various setter methods
+        target = MarkTarget().set_mark(10, 0xffff)
+        self.assertEqual(target.to_iptables_args(), ['MARK', '--set-mark', '0xa/0xffff'])
+        target = MarkTarget().set_xmark(20, 0xffff)
+        self.assertEqual(target.to_iptables_args(), ['MARK', '--set-xmark', '0x14/0xffff'])
+        target = MarkTarget().and_mark(0xffff)
+        self.assertEqual(target.to_iptables_args(), ['MARK', '--and-mark', '0xffff'])
+        target = MarkTarget().xor_mark(0xffff)
+        self.assertEqual(target.to_iptables_args(), ['MARK', '--xor-mark', '0xffff'])
+        target = MarkTarget().or_mark(0xffff)
+        self.assertEqual(target.to_iptables_args(), ['MARK', '--or-mark', '0xffff'])
+
+
+
 class TestPrefix(unittest.TestCase):
     """Test chain prefix handling
     """
